#!/bin/sh

LOGFILE=test.log
SERVERLOG=serv.log
SSLDIR=./test/ssl
OPENSSL=openssl
OPENSSLCONF="-config ./test/openssl/openssl.cnf"
if [ "$RUBY" = "" ]
then
  RUBY=ruby
fi

SCRIPTS="test/bind.rb \
         test/add.rb \
         test/add2.rb \
         test/search.rb \
         test/search2.rb \
         test/search3.rb \
         test/modrdn.rb \
         test/search2.rb \
         test/delete.rb \
	 test/search2.rb"

case "$1" in
'clean')
  rm -rf $LOGFILE $SERVERLOG $SSLDIR
  exit 0
  ;;
'openldap1')
  # for OpenLDAP1
  TESTDIR=./test/openldap1
  LDAPD=/usr/local-1.2.11/libexec/slapd
  LD_LIBRARY_PATH=/usr/local-1.2.11/lib:$LD_LIBRARY_PATH
  CONFIG=$TESTDIR/slapd.conf
  DBDIR=$TESTDIR/test-db
  PIDFILE=$DBDIR/slapd.pid
  PORT=6666
  LDAPDOPT="-d 2 -f $CONFIG -p $PORT"
  LDAPDCMD="$LDAPD $LDAPDOPT"
  ;;
'openldap2')
  # for OpenLDAP2
  TESTDIR=./test/openldap2
  LDAPD=/usr/local/libexec/slapd
  LD_LIBRARY_PATH=/usr/local/lib:$LD_LIBRARY_PATH
  CONFIG=$TESTDIR/slapd.conf
  DBDIR=$TESTDIR/test-db
  PIDFILE=$DBDIR/slapd.pid
  PORT=6666
  LDAPURL="ldap://127.0.0.1:$PORT/"
  LDAPDOPT="-d 2 -f $CONFIG -h \"$LDAPURL\""
  LDAPDCMD="$LDAPD $LDAPDOPT"
  ;;
'openldap2-ssl')
  # -- experimental -- OpenLDAP2 with SSL,SASL
  TESTDIR=./test/openldap2
  LDAPD=/usr/local/libexec/slapd
  LD_LIBRARY_PATH=/usr/local/lib:$LD_LIBRARY_PATH
  CONFIG=$TESTDIR/slapd-ssl.conf
  DBDIR=$TESTDIR/test-db
  PIDFILE=$DBDIR/slapd.pid
  PORT=6666
  SSLPORT=6667
  LDAPURL="ldaps://127.0.0.1:$SSLPORT/ ldap://127.0.0.1:$PORT/"
  LDAPDOPT="-d 3 -f $CONFIG -h \"$LDAPURL\""
  LDAPDCMD="$LDAPD $LDAPDOPT"
  EXT_SCRIPTS="test/bind-ssl.rb"
  ;;
'newcert')
  rm -rf $SSLDIR
  mkdir $SSLDIR
  openssl req -new -x509 -out $SSLDIR/server.pem \
          -nodes -keyout $SSLDIR/server.pem \
	  -config ./test/openssl/openssl.cnf
  exit 0
  ;;
*)
  echo "$0 {openldap1|openldap2|openldap2-ssl(experimental)|newcert|clean}"
  exit
  ;;
esac


error(){
  echo $@ 1>&2
}

runruby(){
  echo "-- $RUBY $1 $PORT $SSLPORT --" >> $LOGFILE
  $RUBY $1 $PORT $SSLPORT 1>> $LOGFILE 2>> $LOGFILE
}


if [ `basename $PWD` = "test" ]; then
  error "run in the parent directory"
  exit
fi

# initialize DB
rm -rf $DBDIR
mkdir $DBDIR

# start slapd
rm -f $LOGFILE
rm -f $SERVERLOG

eval $LDAPDCMD 1>> $SERVERLOG 2>> $SERVERLOG &
echo $LDAPDCMD

while [ ! -f "$PIDFILE" ]; do
  echo "waiting for ldapd"
  sleep 1
done


if [ -f "$LOGFILE" ]; then
  rm $LOGFILE
fi

RESULT=ok
for f in $SCRIPTS
do
  printf "running $f .... "
  if runruby $f; then
    echo "succeed"
  else
    echo "fail"
    RESULT=error
  fi
done

for f in $EXT_SCRIPTS
do
  printf "running $f .... "
  if runruby $f; then
    echo "succeed"
  else
    echo "not supported"
  fi
done

# stop slapd
kill -TERM `cat $PIDFILE`
rm -rf $DBDIR
